@@ -13,14 +13,12 @@ from electrum.keystore import Hardware_KeyStore, parse_xpubkey
 from ..hw_wallet import HW_PluginBase
 from electrum.util import format_satoshis_plain, print_error
 
-
 try:
     import hid
     from btchip.btchipComm import HIDDongleHIDAPI, DongleWait
     from btchip.btchip import btchip
     from btchip.btchipUtils import compress_public_key,format_transaction, get_regular_input_script, get_p2sh_input_script
     from btchip.bitcoinTransaction import bitcoinTransaction
-    from btchip.btchipPersoWizard import StartBTChipPersoDialog
     from btchip.btchipFirmwareWizard import checkFirmware, updateFirmware
     from btchip.btchipException import BTChipException
     BTCHIP = True
@@ -121,8 +119,7 @@ class Ledger_Client():
             except BTChipException, e:
                 if (e.sw == 0x6985):
                     self.dongleObject.dongle.close()
-                    dialog = StartBTChipPersoDialog()
-                    dialog.exec_()
+                    self.handler.get_setup( )
                     # Acquire the new client on the next run
                 else:
                     raise e
@@ -172,6 +169,12 @@ class Ledger_KeyStore(Hardware_KeyStore):
         # device reconnects
         self.force_watching_only = False
         self.signing = False
+        self.cfg = d.get('cfg', {'mode':0,'pair':''})
+        
+    def dump(self):
+        obj = Hardware_KeyStore.dump(self)
+        obj['cfg'] = self.cfg
+        return obj
 
     def get_derivation(self):
         return self.derivation        
@@ -209,18 +212,19 @@ class Ledger_KeyStore(Hardware_KeyStore):
             info = self.get_client().signMessagePrepare(address_path, message)
             pin = ""
             if info['confirmationNeeded']:
-                # TODO : handle different confirmation types. For the time being only supports keyboard 2FA
-                confirmed, p, pin = self.password_dialog()
-                if not confirmed:
-                    raise Exception('Aborted by user')
-                pin = pin.encode()
-                #self.plugin.get_client(self, True, True)
+                pin = self.handler.get_auth( info ) # does the authenticate dialog and returns pin
+                if not pin:
+                    raise UserWarning(_('Cancelled by user'))
+                pin = str(pin).encode()
             signature = self.get_client().signMessageSign(pin)
         except BTChipException, e:
             if e.sw == 0x6a80:
                 self.give_error("Unfortunately, this message cannot be signed by the Ledger wallet. Only alphanumerical messages shorter than 140 characters are supported. Please remove any extra characters (tab, carriage return) and retry.")
             else:
                 self.give_error(e, True)
+        except UserWarning:
+            self.handler.show_error(_('Cancelled by user'))
+            return ''
         except Exception, e:
             self.give_error(e, True)
         finally:
@@ -334,6 +338,7 @@ class Ledger_KeyStore(Hardware_KeyStore):
             firstTransaction = True
             inputIndex = 0
             rawTx = tx.serialize()
+            self.get_client().enableAlternate2fa(False)
             while inputIndex < len(inputs):
                 self.get_client().startUntrustedTransaction(firstTransaction, inputIndex,
                                                             chipInputs, redeemScripts[inputIndex])
@@ -348,44 +353,24 @@ class Ledger_KeyStore(Hardware_KeyStore):
                 if firstTransaction:
                     transactionOutput = outputData['outputData']
                 if outputData['confirmationNeeded']:
-                    # TODO : handle different confirmation types. For the time being only supports keyboard 2FA
+                    outputData['address'] = output
                     self.handler.clear_dialog()
-                    if 'keycardData' in outputData:
-                        pin2 = ""
-                        for keycardIndex in range(len(outputData['keycardData'])):
-                            msg = "Do not enter your device PIN here !\r\n\r\n" + \
-                                "Your Ledger Wallet wants to talk to you and tell you a unique second factor code.\r\n" + \
-                                "For this to work, please match the character between stars of the output address using your security card\r\n\r\n" + \
-                                "Output address : "
-                            for index in range(len(output)):
-                                if index == outputData['keycardData'][keycardIndex]:
-                                    msg = msg + "*" + output[index] + "*"
-                                else:
-                                    msg = msg + output[index]
-                            msg = msg + "\r\n"
-                            confirmed, p, pin = self.password_dialog(msg)
-                            if not confirmed:
-                                raise Exception('Aborted by user')
-                            try:
-                                pin2 = pin2 + chr(int(pin[0], 16))
-                            except:
-                                raise Exception('Invalid PIN character')
-                        pin = pin2
-                    else:
-                        confirmed, p, pin = self.password_dialog()
-                        if not confirmed:
-                            raise Exception('Aborted by user')
-                        pin = pin.encode()
-                        #self.plugin.get_client(self, True, True)
-                    self.handler.show_message("Signing ...")
+                    pin = self.handler.get_auth( outputData ) # does the authenticate dialog and returns pin
+                    if not pin:
+                        raise UserWarning()
+                    if pin != 'paired':
+                        self.handler.show_message(_("Confirmed. Signing Transaction..."))
                 else:
                     # Sign input with the provided PIN
-                    inputSignature = self.get_client().untrustedHashSign(inputsPaths[inputIndex],
-                    pin)
+                    inputSignature = self.get_client().untrustedHashSign(inputsPaths[inputIndex], pin)
                     inputSignature[0] = 0x30 # force for 1.4.9+
                     signatures.append(inputSignature)
                     inputIndex = inputIndex + 1
-                firstTransaction = False
+                if pin != 'paired':
+                    firstTransaction = False
+        except UserWarning:
+            self.handler.show_error(_('Cancelled by user'))
+            return
         except BaseException as e:
             traceback.print_exc(file=sys.stdout)
             self.give_error(e, True)
@@ -412,23 +397,6 @@ class Ledger_KeyStore(Hardware_KeyStore):
            tx.update_signatures(updatedTransaction)
         self.signing = False
 
-    def password_dialog(self, msg=None):
-        if not msg:
-            msg = _("Do not enter your device PIN here !\r\n\r\n" \
-                    "Your Ledger Wallet wants to talk to you and tell you a unique second factor code.\r\n" \
-                    "For this to work, please open a text editor " \
-                    "(on a different computer / device if you believe this computer is compromised) " \
-                    "and put your cursor into it, unplug your Ledger Wallet and plug it back in.\r\n" \
-                    "It should show itself to your computer as a keyboard " \
-                    "and output the second factor along with a summary of " \
-                    "the transaction being signed into the text-editor.\r\n\r\n" \
-                    "Check that summary and then enter the second factor code here.\r\n" \
-                    "Before clicking OK, re-plug the device once more (unplug it and plug it again if you read the second factor code on the same computer)")
-        response = self.handler.get_word(msg)
-        if response is None:
-            return False, None, None
-        return True, response, response
-
 
 class LedgerPlugin(HW_PluginBase):
     libraries_available = BTCHIP
