@@ -342,7 +342,7 @@ class TrustedCoinPlugin(BasePlugin):
         window.wallet.is_billing = False
 
     def show_disclaimer(self, wizard):
-        wizard.set_icon(':icons/trustedcoin.png')
+        wizard.set_icon(':icons/trustedcoin-wizard.png')
         wizard.stack = []
         wizard.confirm_dialog(title='Disclaimer', message='\n\n'.join(DISCLAIMER), run_next = lambda x: wizard.run('choose_seed'))
 
@@ -413,7 +413,7 @@ class TrustedCoinPlugin(BasePlugin):
         wizard.passphrase_dialog(run_next=f) if is_ext else f('')
 
     def restore_choice(self, wizard, seed, passphrase):
-        wizard.set_icon(':icons/trustedcoin.png')
+        wizard.set_icon(':icons/trustedcoin-wizard.png')
         wizard.stack = []
         title = _('Restore 2FA wallet')
         msg = ' '.join([
